/***************************************************************************
 *   Copyright Brian Ledbetter 2001-2003 <brian@shadowcom.net>             *
 *   Copyright Ravikiran Rajagopal 2003                                    *
 *   ravi@ee.eng.ohio-state.edu                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License (version 2) as   *
 *   published by the Free Software Foundation. (The original KSplash/ML   *
 *   codebase (upto version 0.95.3) is BSD-licensed.)                      *
 *                                                                         *
 ***************************************************************************/

#include <qlabel.h>
#include <qpainter.h>
#include <qtimer.h>
#include <qwidget.h>
#include <qwmatrix.h>

#include <kdebug.h>
#include <kpixmap.h>
#include <kpixmapeffect.h>
#include <kwin.h>

#include "icon.h"
#include "icon.moc"

// Largest possible size for this icon is given to us.
Icon::Icon( const QPixmap& icon, unsigned int min_size, unsigned int max_size, bool optimize )
    : QLabel( 0L, 0L, WStyle_Customize|WStyle_StaysOnTop|WStyle_NoBorderEx ),
    mIcon( icon ), mMinSize( min_size ), mMaxSize( max_size ), mOptimize( optimize ), mInitialized( false )
{
  resize( 0, 0 );
  KWin::setState( winId(), NET::Modal | NET::StaysOnTop | NET::SkipTaskbar | NET::SkipPager );
  KWin::setType( winId(), NET::Dock );
  scalePixmapsForCache();
}

Icon::~Icon()
{
  mIcons.clear();
}

//Prepare pixmaps for each icon size between 16 and 48 and store them in our pixmap cache.
void Icon::scalePixmapsForCache()
{
  // Add a nifty gradient to the background of the icon.  This should make us
  // look like early Mac OS X betas.
  // (While we're at it, let's add a slight margin around each icon.  This ought to
  // make it look a bit cleaner...  Maybe it will even be configurable eventually...)
  KPixmap t;
  t.resize( mIcon.size().width()+4, mIcon.size().height()+4 );
  KPixmapEffect::gradient( t, Qt::white, Qt::gray, KPixmapEffect::VerticalGradient );
  QPainter p;
  p.begin(&t);
  p.drawPixmap( 2, 2, mIcon );
  p.end();
  mIcon = t;

  if( mIcon.width() < mMaxSize )
  {
    QPixmap tmp;
    tmp.resize( mMaxSize, mMaxSize );
    QWMatrix m;
    float f = (float)tmp.width() / mIcon.width();
    m.scale( f, f );
    tmp = mIcon.xForm( m );
    mIcons.append( new QPixmap( tmp ) );
  }
  else
    mIcons.append( new QPixmap( mIcon ) );

  // For each icon size desired, scale the previous icon down by one pixel.
  QWMatrix matrix;
  QPixmap tmp;
  float xscale;

  for( unsigned int i = mMaxSize - 1; i >= mMinSize; i-- )
  {
    xscale = 0.98;
    tmp.resize(i,i);
    matrix.scale( xscale, xscale );
    tmp = mIcons.at( (mMaxSize-i)-1 )->xForm( matrix );
    mIcons.append( new QPixmap( tmp ) );
  }
  mInitialized = true;
}

void Icon::resizeEvent( QResizeEvent *qre )
{
  QLabel::resizeEvent( qre );
  if( mInitialized )
    emit sizeChanged();
}

/*
 * Icon::_incrementalShow(): Slowly show this icon.  We've already set QLabel::show(),
 * so all we need to worry about doing here is move()ing and resize()ing.  (And, since
 * we set QLabel::setAutoResize( true ), we don't even have to do the latter!)
 */
void Icon::incrementalShow()
{
  mSize+=8;
  if ( mSize >= mMaxSize )
  {
    setPixmap( *mIcons.at( 0 ) );
    resize( sizeHint() );
    return;
  }
  setPixmap( *mIcons.at( ( mMaxSize - mSize ) - 1 ) );
  resize( sizeHint() );
  QTimer::singleShot( 100, this, SLOT(incrementalShow()) );
}

/*
 * Icon::showEvent(): The icon has been shown.  We will be really fancy
 * with this - We're going to gradually "zoom" in.  Our IconBar should
 * automatically adjust our position based on this.
 */
void Icon::showEvent( QShowEvent * )
{
  if ( mIcons.isEmpty() )
    return;
  mSize = mMinSize;
  QLabel::show();
  QTimer::singleShot( 100, this, SLOT(incrementalShow()) );
}
